@@ -54,12 +54,13 @@ CREATE TABLE core (
 INSERT INTO core VALUES ('event_fetch_timer', 15);
 INSERT INTO core VALUES ('event_action_timer', 3);
 INSERT INTO core VALUES ('event_display_timer', 4);
-INSERT INTO core VALUES ('time_zone', 'America/Edmonton');
+INSERT INTO core VALUES ('time_zone', 'America/Vancouver');
 INSERT INTO core VALUES ('sensor_pin', -1);
 INSERT INTO core VALUES ('testing', 0);
 INSERT INTO core VALUES ('debug_sensor', 0);
 INSERT INTO core VALUES ('log_file', "");
-INSERT INTO core VALUES ('log_level', -1);
+INSERT INTO core VALUES ('debug_level', -1);
+INSERT INTO core VALUES ('devel', 0);
 
 DROP TABLE IF EXISTS light;
 
